@@ -66,14 +66,14 @@ pub(crate) fn stdin_loop(
                 && stdin_buffer
                     .iter()
                     .take(bracketed_paste_start.len())
-                    .eq(bracketed_paste_start.iter()))
+                    .eq(&bracketed_paste_start))
         {
             match bracketed_paste_end_position(&stdin_buffer) {
                 Some(paste_end_position) => {
                     let starts_with_bracketed_paste_start = stdin_buffer
                         .iter()
                         .take(bracketed_paste_start.len())
-                        .eq(bracketed_paste_start.iter());
+                        .eq(&bracketed_paste_start);
 
                     let ends_with_bracketed_paste_end = true;
 
@@ -94,8 +94,20 @@ pub(crate) fn stdin_loop(
                     pasting = false;
                 }
                 None => {
+                    let starts_with_bracketed_paste_start = stdin_buffer
+                        .iter()
+                        .take(bracketed_paste_start.len())
+                        .eq(&bracketed_paste_start);
+                    if starts_with_bracketed_paste_start {
+                        drop(stdin_buffer.drain(..6)); // bracketed paste start
+                    }
+
                     send_input_instructions
-                        .send(InputInstruction::PastedText((true, stdin_buffer, false)))
+                        .send(InputInstruction::PastedText((
+                            starts_with_bracketed_paste_start,
+                            stdin_buffer,
+                            false,
+                        )))
                         .unwrap();
                     pasting = true;
                     continue;
