@@ -1,3 +1,4 @@
+use std::fmt::Write;
 use std::sync::mpsc::channel;
 use std::time::Instant;
 use std::unimplemented;
@@ -11,7 +12,7 @@ use crate::ClientId;
 use zellij_utils::pane_size::Offset;
 use zellij_utils::position::Position;
 use zellij_utils::shared::ansi_len;
-use zellij_utils::zellij_tile::prelude::{Event, Mouse, PaletteColor};
+use zellij_utils::zellij_tile::prelude::{Event, InputMode, Mouse, PaletteColor};
 use zellij_utils::{
     channels::SenderWithContext,
     pane_size::{Dimension, PaneGeom},
@@ -27,6 +28,7 @@ pub(crate) struct PluginPane {
     pub send_plugin_instructions: SenderWithContext<PluginInstruction>,
     pub active_at: Instant,
     pub pane_title: String,
+    pub pane_name: String,
     frame: bool,
     borderless: bool,
 }
@@ -37,6 +39,7 @@ impl PluginPane {
         position_and_size: PaneGeom,
         send_plugin_instructions: SenderWithContext<PluginInstruction>,
         title: String,
+        pane_name: String,
     ) -> Self {
         Self {
             pid,
@@ -50,6 +53,7 @@ impl PluginPane {
             content_offset: Offset::default(),
             pane_title: title,
             borderless: false,
+            pane_name,
         }
     }
 }
@@ -130,7 +134,10 @@ impl Pane for PluginPane {
     fn set_selectable(&mut self, selectable: bool) {
         self.selectable = selectable;
     }
-    fn render(&mut self) -> Option<String> {
+    fn render(&mut self, client_id: Option<ClientId>) -> Option<String> {
+        // this is a bit of a hack but works in a pinch
+        client_id?;
+        let client_id = client_id.unwrap();
         // if self.should_render {
         if true {
             // while checking should_render rather than rendering each pane every time
@@ -144,6 +151,7 @@ impl Pane for PluginPane {
                 .send(PluginInstruction::Render(
                     buf_tx,
                     self.pid,
+                    client_id,
                     self.get_content_rows(),
                     self.get_content_columns(),
                 ))
@@ -165,12 +173,14 @@ impl Pane for PluginPane {
                     .concat()
                 };
 
-                vte_output.push_str(&format!(
+                write!(
+                    &mut vte_output,
                     "\u{1b}[{};{}H\u{1b}[m{}",
                     self.get_content_y() + 1 + index,
                     self.get_content_x() + 1,
                     line_to_print,
-                )); // goto row/col and reset styles
+                )
+                .unwrap(); // goto row/col and reset styles
                 let line_len = line_to_print.len();
                 if line_len < self.get_content_columns() {
                     // pad line
@@ -185,11 +195,13 @@ impl Pane for PluginPane {
                 for line_index in total_line_count..self.get_content_rows() {
                     let x = self.get_content_x();
                     let y = self.get_content_y();
-                    vte_output.push_str(&format!(
+                    write!(
+                        &mut vte_output,
                         "\u{1b}[{};{}H\u{1b}[m",
                         y + line_index + 1,
                         x + 1
-                    )); // goto row/col and reset styles
+                    )
+                    .unwrap(); // goto row/col and reset styles
                     for _col_index in 0..self.get_content_columns() {
                         vte_output.push(' ');
                     }
@@ -200,14 +212,29 @@ impl Pane for PluginPane {
             None
         }
     }
-    fn render_frame(&mut self, _client_id: ClientId, frame_params: FrameParams) -> Option<String> {
+    fn render_frame(
+        &mut self,
+        _client_id: ClientId,
+        frame_params: FrameParams,
+        input_mode: InputMode,
+    ) -> Option<String> {
         // FIXME: This is a hack that assumes all fixed-size panes are borderless. This
         // will eventually need fixing!
         if self.frame && !(self.geom.rows.is_fixed() || self.geom.cols.is_fixed()) {
+            let pane_title = if self.pane_name.is_empty()
+                && input_mode == InputMode::RenamePane
+                && frame_params.is_main_client
+            {
+                String::from("Enter name...")
+            } else if self.pane_name.is_empty() {
+                self.pane_title.clone()
+            } else {
+                self.pane_name.clone()
+            };
             let frame = PaneFrame::new(
                 self.current_geom().into(),
                 (0, 0), // scroll position
-                self.pane_title.clone(),
+                pane_title,
                 frame_params,
             );
             Some(frame.render())
@@ -222,6 +249,20 @@ impl Pane for PluginPane {
     ) -> Option<String> {
         None
     }
+    fn update_name(&mut self, name: &str) {
+        match name {
+            "\0" => {
+                self.pane_name = String::new();
+            }
+            "\u{007F}" | "\u{0008}" => {
+                //delete and backspace keys
+                self.pane_name.pop();
+            }
+            c => {
+                self.pane_name.push_str(c);
+            }
+        }
+    }
     fn pid(&self) -> PaneId {
         PaneId::Plugin(self.pid)
     }
@@ -269,6 +310,7 @@ impl Pane for PluginPane {
         self.send_plugin_instructions
             .send(PluginInstruction::Update(
                 Some(self.pid),
+                None,
                 Event::Mouse(Mouse::ScrollUp(count)),
             ))
             .unwrap();
@@ -277,6 +319,7 @@ impl Pane for PluginPane {
         self.send_plugin_instructions
             .send(PluginInstruction::Update(
                 Some(self.pid),
+                None,
                 Event::Mouse(Mouse::ScrollDown(count)),
             ))
             .unwrap();
@@ -288,6 +331,7 @@ impl Pane for PluginPane {
         self.send_plugin_instructions
             .send(PluginInstruction::Update(
                 Some(self.pid),
+                None,
                 Event::Mouse(Mouse::LeftClick(start.line.0, start.column.0)),
             ))
             .unwrap();
@@ -296,6 +340,7 @@ impl Pane for PluginPane {
         self.send_plugin_instructions
             .send(PluginInstruction::Update(
                 Some(self.pid),
+                None,
                 Event::Mouse(Mouse::Hold(position.line.0, position.column.0)),
             ))
             .unwrap();
@@ -304,6 +349,7 @@ impl Pane for PluginPane {
         self.send_plugin_instructions
             .send(PluginInstruction::Update(
                 Some(self.pid),
+                None,
                 Event::Mouse(Mouse::Release(
                     end.map(|Position { line, column }| (line.0, column.0)),
                 )),
@@ -337,6 +383,7 @@ impl Pane for PluginPane {
         self.send_plugin_instructions
             .send(PluginInstruction::Update(
                 Some(self.pid),
+                None,
                 Event::Mouse(Mouse::RightClick(to.line.0, to.column.0)),
             ))
             .unwrap();
