@@ -9,7 +9,7 @@ use crate::pty::VteBytes;
 use crate::tab::Pane;
 use crate::ClientId;
 use std::collections::{HashMap, HashSet};
-use std::fmt::Debug;
+use std::fmt::{Debug, Write};
 use std::os::unix::io::RawFd;
 use std::time::{self, Instant};
 use zellij_utils::pane_size::Offset;
@@ -17,7 +17,7 @@ use zellij_utils::{
     pane_size::{Dimension, PaneGeom},
     position::Position,
     vte,
-    zellij_tile::data::{Palette, PaletteColor},
+    zellij_tile::data::{InputMode, Palette, PaletteColor},
 };
 
 pub const SELECTION_SCROLL_INTERVAL_MS: u64 = 10;
@@ -44,6 +44,7 @@ pub struct TerminalPane {
     selection_scrolled_at: time::Instant,
     content_offset: Offset,
     pane_title: String,
+    pane_name: String,
     frame: HashMap<ClientId, PaneFrame>,
     borderless: bool,
     fake_cursor_locations: HashSet<(usize, usize)>, // (x, y) - these hold a record of previous fake cursors which we need to clear on render
@@ -93,8 +94,8 @@ impl Pane for TerminalPane {
         self.reflow_lines();
     }
     fn handle_pty_bytes(&mut self, bytes: VteBytes) {
-        for byte in bytes.iter() {
-            self.vte_parser.advance(&mut self.grid, *byte);
+        for &byte in &bytes {
+            self.vte_parser.advance(&mut self.grid, byte);
         }
         self.set_should_render(true);
     }
@@ -183,7 +184,8 @@ impl Pane for TerminalPane {
     fn set_selectable(&mut self, selectable: bool) {
         self.selectable = selectable;
     }
-    fn render(&mut self) -> Option<String> {
+    fn render(&mut self, _client_id: Option<ClientId>) -> Option<String> {
+        // we don't use client_id because terminal panes render the same for all users
         if self.should_render() {
             let mut vte_output = String::new();
             let mut character_styles = CharacterStyles::new();
@@ -191,11 +193,13 @@ impl Pane for TerminalPane {
             let content_y = self.get_content_y();
             if self.grid.clear_viewport_before_rendering {
                 for line_index in 0..self.grid.height {
-                    vte_output.push_str(&format!(
+                    write!(
+                        &mut vte_output,
                         "\u{1b}[{};{}H\u{1b}[m",
                         content_y + line_index + 1,
                         content_x + 1
-                    )); // goto row/col and reset styles
+                    )
+                    .unwrap(); // goto row/col and reset styles
                     for _col_index in 0..self.grid.width {
                         vte_output.push(EMPTY_TERMINAL_CHARACTER.character);
                     }
@@ -212,8 +216,14 @@ impl Pane for TerminalPane {
                 self.grid.update_line_for_rendering(y);
                 let x = content_x + x;
                 let y = content_y + y;
-                vte_output.push_str(&format!("\u{1b}[{};{}H\u{1b}[m", y + 1, x + 1));
-                vte_output.push(EMPTY_TERMINAL_CHARACTER.character);
+                write!(
+                    &mut vte_output,
+                    "\u{1b}[{};{}H\u{1b}[m{}",
+                    y + 1,
+                    x + 1,
+                    EMPTY_TERMINAL_CHARACTER.character
+                )
+                .unwrap();
             }
             let max_width = self.get_content_columns();
             for character_chunk in self.grid.read_changes() {
@@ -222,11 +232,13 @@ impl Pane for TerminalPane {
                 let chunk_absolute_x = pane_x + character_chunk.x;
                 let chunk_absolute_y = pane_y + character_chunk.y;
                 let terminal_characters = character_chunk.terminal_characters;
-                vte_output.push_str(&format!(
+                write!(
+                    &mut vte_output,
                     "\u{1b}[{};{}H\u{1b}[m",
                     chunk_absolute_y + 1,
                     chunk_absolute_x + 1
-                )); // goto row/col and reset styles
+                )
+                .unwrap(); // goto row/col and reset styles
 
                 let mut chunk_width = character_chunk.x;
                 for mut t_character in terminal_characters {
@@ -248,9 +260,13 @@ impl Pane for TerminalPane {
                     if let Some(new_styles) = character_styles
                         .update_and_return_diff(&t_character.styles, self.grid.changed_colors)
                     {
-                        vte_output.push_str(&new_styles.to_string());
-                        vte_output
-                            .push_str(&self.grid.link_handler.output_osc8(new_styles.link_anchor))
+                        write!(
+                            &mut vte_output,
+                            "{}{}",
+                            new_styles,
+                            self.grid.link_handler.output_osc8(new_styles.link_anchor)
+                        )
+                        .unwrap();
                     }
 
                     vte_output.push(t_character.character);
@@ -263,16 +279,31 @@ impl Pane for TerminalPane {
             None
         }
     }
-    fn render_frame(&mut self, client_id: ClientId, frame_params: FrameParams) -> Option<String> {
+    fn render_frame(
+        &mut self,
+        client_id: ClientId,
+        frame_params: FrameParams,
+        input_mode: InputMode,
+    ) -> Option<String> {
         // TODO: remove the cursor stuff from here
         let mut vte_output = None;
-        let frame = PaneFrame::new(
-            self.current_geom().into(),
-            self.grid.scrollback_position_and_length(),
+        let pane_title = if self.pane_name.is_empty()
+            && input_mode == InputMode::RenamePane
+            && frame_params.is_main_client
+        {
+            String::from("Enter name...")
+        } else if self.pane_name.is_empty() {
             self.grid
                 .title
                 .clone()
-                .unwrap_or_else(|| self.pane_title.clone()),
+                .unwrap_or_else(|| self.pane_title.clone())
+        } else {
+            self.pane_name.clone()
+        };
+        let frame = PaneFrame::new(
+            self.current_geom().into(),
+            self.grid.scrollback_position_and_length(),
+            pane_title,
             frame_params,
         );
         match self.frame.get(&client_id) {
@@ -321,6 +352,20 @@ impl Pane for TerminalPane {
         }
         vte_output
     }
+    fn update_name(&mut self, name: &str) {
+        match name {
+            "\0" => {
+                self.pane_name = String::new();
+            }
+            "\u{007F}" | "\u{0008}" => {
+                //delete and backspace keys
+                self.pane_name.pop();
+            }
+            c => {
+                self.pane_name.push_str(c);
+            }
+        }
+    }
     fn pid(&self) -> PaneId {
         PaneId::Terminal(self.pid)
     }
@@ -460,6 +505,7 @@ impl TerminalPane {
         position_and_size: PaneGeom,
         palette: Palette,
         pane_index: usize,
+        pane_name: String,
     ) -> TerminalPane {
         let initial_pane_title = format!("Pane #{}", pane_index);
         let grid = Grid::new(
@@ -480,6 +526,7 @@ impl TerminalPane {
             colors: palette,
             selection_scrolled_at: time::Instant::now(),
             pane_title: initial_pane_title,
+            pane_name,
             borderless: false,
             fake_cursor_locations: HashSet::new(),
         }
