@@ -74,7 +74,7 @@ pub(crate) struct Pty {
 
 use std::convert::TryFrom;
 
-pub(crate) fn pty_thread_main(mut pty: Pty, layout: LayoutFromYaml) {
+pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<LayoutFromYaml>) {
     loop {
         let (event, mut err_ctx) = pty.bus.recv().expect("failed to receive event on channel");
         err_ctx.add_call(ContextType::Pty((&event).into()));
@@ -284,13 +284,12 @@ impl Pty {
                         PaneId::Plugin(..) => None,
                         PaneId::Terminal(id) => self.id_to_child_pid.get(id),
                     })
-                    .and_then(|id| {
+                    .and_then(|&id| {
                         self.bus
                             .os_input
                             .as_ref()
-                            .map(|input| input.get_cwd(Pid::from_raw(*id)))
-                    })
-                    .flatten();
+                            .and_then(|input| input.get_cwd(Pid::from_raw(id)))
+                    });
             };
         };
     }
