@@ -1,3 +1,5 @@
 pub mod boundaries;
+pub mod overlay;
 pub mod pane_boundaries_frame;
+pub mod pane_contents_and_ui;
 pub mod pane_resizer;
